{-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}

{-# LANGUAGE TypeApplications #-}

module Test.Ngrams.Count (tests) where

import Gargantext.API.Ngrams
import Gargantext.Core (Lang(..))
import Gargantext.Core.Text.Terms.WithList (buildPatternsWith, termsInText, Pattern(..))
import Gargantext.Prelude
import Test.Tasty
import Test.Tasty.HUnit


tests :: TestTree
tests = testGroup "Ngrams" [unitTests]

unitTests :: TestTree
unitTests = testGroup "Count tests"
  [ -- Sorting
    testCase "Build patterns works 01" testBuildPatterns01
  , testCase "Build patterns works 02" testBuildPatterns02
  , testCase "termsInText works 01" testTermsInText01
  , testCase "termsInText works 02" testTermsInText02
  , testCase "termsInText works 03" testTermsInText03
  , testCase "termsInText works 04 (related to issue #221)" testTermsInText04
  ]

-- | Let's document how the `buildPatternsWith` function works.
testBuildPatterns01 :: Assertion
testBuildPatterns01 = do
  let terms = ["hello world"] :: [NgramsTerm]
  let enPatterns = buildPatternsWith EN terms
  length enPatterns @?= 1
  let Just pat = head enPatterns
  _pat_length pat @?= 2
  _pat_terms pat @?= ["hello", "world"]

-- | Let's document how the `buildPatternsWith` function works.
testBuildPatterns02 :: Assertion
testBuildPatterns02 = do
  let terms = ["hello", "world"] :: [NgramsTerm]
  let enPatterns = buildPatternsWith EN terms
  length enPatterns @?= 2
  let [pat1, pat2] = enPatterns
  _pat_length pat1 @?= 1
  _pat_terms pat1 @?= ["hello"]
  _pat_length pat2 @?= 1
  _pat_terms pat2 @?= ["world"]

-- | Let's document how the `termsInText` function works.
testTermsInText01 :: Assertion
testTermsInText01 = do
  let terms = ["hello", "world"] :: [NgramsTerm]
  let enPatterns = buildPatternsWith EN terms
  let tit = termsInText EN enPatterns "Hello, world!"
  length tit @?= 2
  let [tit1, tit2] = tit
  tit1 @?= ("hello", 1)
  tit2 @?= ("world", 1)

-- | Let's document how the `termsInText` function works.
testTermsInText02 :: Assertion
testTermsInText02 = do
  let terms = ["hello", "world"] :: [NgramsTerm]
  let enPatterns = buildPatternsWith EN terms
  let tit = termsInText EN enPatterns "Hello, world, hello!"
  length tit @?= 2
  let [tit1, tit2] = tit
  tit1 @?= ("hello", 2)
  tit2 @?= ("world", 1)

-- | Let's document how the `termsInText` function works.
testTermsInText03 :: Assertion
testTermsInText03 = do
  let terms = ["hello", "world"] :: [NgramsTerm]
  let enPatterns = buildPatternsWith EN terms
  let tit = termsInText EN enPatterns "Hello, world, again!"
  length tit @?= 2
  let [tit1, tit2] = tit
  tit1 @?= ("hello", 1)
  tit2 @?= ("world", 1)

-- | Let's document how the `termsInText` function works.
-- https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/221
testTermsInText04 :: Assertion
testTermsInText04 = do
  let terms = ["feuilles de basilic"] :: [NgramsTerm]
  let frPatterns = buildPatternsWith FR terms
  let tit = termsInText FR frPatterns "Infos pratiques Nombre de personnes 1 personne Quantité1 verre Temps de préparation 5 minutes Degré de difficulté Très facile Coût Abordable Les ingrédients de la recette 4 feuilles de basilic 1 branche de romarin 15 ml de citron jaune 60 ml d'eau gazeuse au mastiqua 90 ml de Bulles de Muscat Jaillance La préparation de la recette Verser dans un verre type long drink le citron jaune, les feuilles de basilic et l'eau gazeuse."
  length tit @?= 1
  let [tit1] = tit
  tit1 @?= ("feuilles de basilic", 2)
